/*
*  Rubrica
*  file: 
*
*  
*  Copyright (C) 2000-2005 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include <gtk/gtk.h>
#include <glade/glade.h>

#include "types.h"
#include "gui_types.h"

#include "app.h"
#include "adb.h"
#include "models.h"
#include "callbacks.h"
#include "groups_cb.h"
#include "popmenu.h"
#include "utils.h"
#include "view.h"


typedef struct {
  const gchar* pixmap;
  const gchar* label;
  gint items;
  gint page;
} RubricaAddressesTree;


RubricaAddressesTree address_tree[] = {
  {"rubrica.png",    "Addressbooks", 0, -1},
  {"gtk-delete.png", "Trash",        0, 0},
  {NULL, NULL, 0}
};


static RubricaDefaultCategories defcats[] = {
  {"",                 "no group"   },
  {"all.png",          "all groups" },
  {"favorites.png",    "favorites"  },
  {"hot-contacts.png", "hot contact"},
  {"friends.png",      "friends"    },
  {"heart.png",        "heart"      },
  {"home.png",         "relatives"  },
  {"holiday.png",      "holidays"   },
  {"hobby.png",        "hobbies"    },
  {"football.png",     "sport"      },
  {"money.png",        "bussines"   },
  {"company2.png",     "company"    },
  {"genius.png",       "work"       },
  {"customer.png",     "customer"   },
  {"strategies.png",   "strategies" },
  {"penguin.png",      "programmers"},
  {"freebsd.png",      "BSD friends"},
  {"university.png",   "university" },
  {"pj.png",           "journal"    },
  {NULL, NULL}
};


typedef struct _RubricaMenu {
  GtkWidget* menubar;
  GtkWidget* new;
  GtkWidget* open;
  GtkWidget* save;
  GtkWidget* saveas;
  GtkWidget* quit;

  GtkWidget* cut;
  GtkWidget* copy;
  GtkWidget* paste;

  GtkWidget* personal;  
  GtkWidget* company;
  GtkWidget* modify;
  GtkWidget* delete;

  GtkWidget* prefer;

  GtkWidget* about;
} RubricaMenu;


typedef struct _RubricaToolbar {
  GtkWidget* handle;
  GtkWidget* toolbar;

  GtkWidget* new;
  GtkWidget* open;
  GtkWidget* save;
  GtkWidget* print;
  GtkWidget* close;

  GtkWidget* add;
  GtkWidget* choose;
  GtkWidget* delete;
  GtkWidget* modify;

  GtkWidget* first;
  GtkWidget* last;
  GtkWidget* search;
  GtkWidget* quit;
} RubricaToolbar;


typedef struct _RubricaDesk {
  GtkWidget* groupslist;            // addressbooks's groups
  GtkWidget* bookstree;             // opened addressbooks
  GtkWidget* cardslist;             // addressbook

  GtkWidget* notebook;             // notebook that contains addressbooks

  GtkWidget* textview;
  
  GtkWidget* previous;
  GtkWidget* next;
  GtkWidget* statusbar;

  GtkWidget* smile;                 // rank image
  GtkWidget* lock;                  // lock image
} RubricaDesk;



/*    signals enumeration 
 */
// static guint rubrica_app_signals[LAST_SIGNAL] = {0};


struct _RubricaAppPrivate {
  GtkWidget       *window;

  RubricaAdb      *active;         // active addressbook
  RubricaAdb      *trash;          // trash
  RubricaGroupSet *set;            // rubrica's groups

  RubricaMenu     *menu;
  RubricaToolbar  *toolbar;
  RubricaDesk     *desk;  
  
  GList* stack;  
  
  guint context_id;
  
  gboolean dispose_has_run;
};




static void rubrica_app_class_init (RubricaAppClass* klass);
static void rubrica_app_init       (RubricaApp* obj);

static void rubrica_app_dispose    (RubricaApp* obj);
static void rubrica_app_finalize   (RubricaApp* obj);


/*   Private
*/
static RubricaMenu*    rubrica_app_build_menu    (GladeXML* gui,
						  RubricaApp* app);
static RubricaToolbar* rubrica_app_build_toolbar (GladeXML* gui, 
						  RubricaApp* app);
static RubricaDesk*    rubrica_app_build_desk    (GladeXML* gui, 
						  RubricaApp* app);

static GtkWidget*      cards_list_new            (RubricaApp* app, 
						  RubricaAdb* adb);
static void            append_addresses_branches (RubricaApp* app, 
						  GtkTreeView  *view);
static void            init_trash_branch         (RubricaApp* app);
static void            update_addresses_tree     (RubricaApp* app, 
						  gchar* name, gint page);

static void            on_update_bookstree       (RubricaAbook* book,
						  gpointer data);
static void            on_show_menu_cb           (GtkMenuToolButton *mtbutton, 
						  gpointer data);
static void            append_personal_card      (GtkTreeView *view,
						  RubricaPersonalCard* card);
static void            append_company_card       (GtkTreeView *view,
						  RubricaCompanyCard* card);



/*  Private
*/

static GtkWidget* 
cards_list_new(RubricaApp* app, RubricaAdb* adb)
{
  GtkTreeSelection *selection;
  GtkTreeView  *cards_list;
  GtkTreeModel *model;
  GtkWidget    *box;
  GtkWidget    *scrollwin;

  g_return_val_if_fail(IS_RUBRICA_APP(app), NULL);
  g_return_val_if_fail(IS_RUBRICA_ADB(adb), NULL);

  box = gtk_vbox_new(FALSE, 6);   // notebook's child

  /* cards list's container
   */
  scrollwin = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwin),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrollwin),
				      GTK_SHADOW_IN);
  gtk_box_pack_start(GTK_BOX(box), scrollwin, TRUE, TRUE, 0);
    

  /*    cards list view (addresses in address_adb)
   */
  cards_list = (GtkTreeView*) gtk_tree_view_new();
  model      = rubrica_model_new(CARDS_MODEL);
  selection  = gtk_tree_view_get_selection(GTK_TREE_VIEW(cards_list));

  gtk_tree_view_set_model(GTK_TREE_VIEW(cards_list), GTK_TREE_MODEL(model));
  gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
  g_object_unref(model);

  gtk_container_add(GTK_CONTAINER(scrollwin), (GtkWidget*) cards_list);

  rubrica_model_add_columns(GTK_TREE_VIEW(cards_list), CARDS_COLUMNS);
  rubrica_adb_set_tree(adb, cards_list);
  
  gtk_widget_show(box);
  gtk_widget_show(scrollwin);
  gtk_widget_show(GTK_WIDGET(cards_list));

  /*   Handle signals
   */
  g_signal_connect(GTK_OBJECT (cards_list), "button_press_event",
		   G_CALLBACK (on_card_tree_event_occurred), app);  

  return box;
}



static void 
append_addresses_branches(RubricaApp* app, GtkTreeView  *view)
{
  GtkTreeModel *model;
  GdkPixbuf    *pixbuf = NULL;
  GtkTreeIter   iter;  
  GtkTreeSelection  *selection;
  RubricaAddressesTree* tree = address_tree;

  g_return_if_fail(IS_RUBRICA_APP(app));

  model     = rubrica_model_new(ADDRESSBOOK_MODEL); 
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view)); 

  gtk_tree_view_set_model(GTK_TREE_VIEW(view), model); 
  gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
  
  rubrica_model_add_columns (GTK_TREE_VIEW(view), ADDRESSBOOK_COLUMNS);
  
  for (; tree->label; tree++)
    { 
      gchar* file;
      
      file = g_strdup_printf("%s/main/%s", RUBRICA_GUI_DIR,
			     tree->pixmap);				 
      pixbuf = gdk_pixbuf_new_from_file_at_size(file, 20, 20, NULL); 
      g_free(file);

      gtk_tree_store_append(GTK_TREE_STORE(model), &iter, NULL);
      gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
			 ADDRESSBOOK_PIXMAP_COLUMN,   pixbuf,
			 ADDRESSBOOK_FILENAME_COLUMN, _(tree->label),
			 ADDRESSBOOK_PAGE_COLUMN,     tree->page,
			 -1);
    }
  
  g_signal_connect(G_OBJECT(selection), "changed", 
		   G_CALLBACK(on_selection_changed), app);
}


static void 
init_trash_branch (RubricaApp* app)
{
  GtkTreeView  *trash_tree;
  GtkTreeStore *model;
  GtkTreeIter  iter; 
  GtkWidget    *child;
  GtkWidget    *label;
  gint page;

  g_return_if_fail(IS_RUBRICA_APP(app));

  app->private->trash = rubrica_adb_new();
  
  child = cards_list_new(app, app->private->trash);
  g_object_set_data(G_OBJECT(child), "Trash", app->private->trash);  
  
  /*    append new card's list to notebook
  */
  label = gtk_label_new("Trash");
  page = gtk_notebook_append_page(GTK_NOTEBOOK(app->private->desk->notebook), 
				  child, label);

  /*    Address Tree branch
  */  
  trash_tree = (GtkTreeView*) app->private->desk->bookstree;
  model      = (GtkTreeStore*) gtk_tree_view_get_model(trash_tree);
  
  if (gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(model), &iter, "1"))
    gtk_tree_store_set(model, &iter, ADDRESSBOOK_PAGE_COLUMN, page, -1);

  g_object_unref(model);
}


static void 
update_addresses_tree (RubricaApp* app, gchar* name, gint page)
{
  GtkTreeView  *view;
  GtkTreeModel *model;
  GdkPixbuf    *pixbuf = NULL;
  GtkTreeIter   iter, child;    
  GtkTreeSelection *select;
  GtkTreePath* path;

  g_return_if_fail(IS_RUBRICA_APP(app));

  view   = (GtkTreeView*) app->private->desk->bookstree;
  model  = gtk_tree_view_get_model(view);
  select = gtk_tree_view_get_selection(view);

  pixbuf = rubrica_utils_pixbuf_new ("ledgreen.png");

  if (gtk_tree_model_get_iter_first(model, &iter))
    {      
      gtk_tree_store_append(GTK_TREE_STORE(model), &child, &iter);
      gtk_tree_store_set(GTK_TREE_STORE(model), &child,
			 ADDRESSBOOK_PIXMAP_COLUMN,   pixbuf,
			 ADDRESSBOOK_FILENAME_COLUMN, name,
			 ADDRESSBOOK_PAGE_COLUMN,     page,
			 -1);
      
      path = gtk_tree_model_get_path(model, &child);
      gtk_tree_selection_select_path(select, path);
      gtk_tree_path_free(path); 
    }
}


static void 
append_personal_card(GtkTreeView *view, RubricaPersonalCard* card)
{
  RubricaInfos *infos;
  GtkTreeModel *model;
  GtkTreeIter  iter;
  
  g_return_if_fail(IS_RUBRICA_PERSONAL_CARD(card));

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(view));  
  infos = rubrica_card_get_infos(RUBRICA_CARD(card));
    
  gtk_list_store_append(GTK_LIST_STORE(model), &iter);
  gtk_list_store_set(GTK_LIST_STORE(model), &iter,
		     CARD_COLUMN,       rubrica_infos_get_label(infos), 
		     FIRST_NAME_COLUMN, rubrica_infos_get_first(infos), 
		     LAST_NAME_COLUMN,  rubrica_infos_get_last(infos),
		     PROFESSION_COLUMN, rubrica_infos_get_profession(infos),
		     CITY_COLUMN,       rubrica_infos_get_city(infos),
		     COUNTRY_COLUMN,    rubrica_infos_get_country(infos),
		     WEB_CARD_COLUMN,   rubrica_infos_get_web(infos),
		     EMAIL_CARD_COLUMN, rubrica_infos_get_email(infos),
		     TELEPHONE_COLUMN,  rubrica_infos_get_telephone(infos),
		     ASSIGMENT_COLUMN,  rubrica_infos_get_assignment(infos),
		     ID_COLUMN,         rubrica_infos_get_id(infos),
		     -1); 

  rubrica_card_free_infos(infos);
}


static void 
append_company_card (GtkTreeView *view, RubricaCompanyCard* card)
{
}


static void 
on_show_menu_cb (GtkMenuToolButton *menutoolbutton, gpointer data)
{
  GtkMenu* menu = data;

  gtk_menu_popup (menu, NULL, NULL, NULL, NULL, 1, 
		  gtk_get_current_event_time());    

  gtk_widget_show_all(GTK_WIDGET(menu));  
}


static void 
on_update_bookstree(RubricaAbook* book, gpointer data)
{
  RubricaApp* app = (RubricaApp*) data;
  
  rubrica_app_update_bookstree_view(app);
}




GType
rubrica_app_get_type()
{
  static GType app_type = 0;
  
  if (!app_type)
    {
      static const GTypeInfo app_info =
	{
	  sizeof(RubricaAppClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_app_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaApp),
	  0,
	  (GInstanceInitFunc) rubrica_app_init
	};

      app_type = g_type_register_static (G_TYPE_OBJECT, "RubricaApp",
					 &app_info, 0);
    }
  
  return app_type;
}



static void
rubrica_app_class_init(RubricaAppClass* klass)
{
  GObjectClass *class; 

  class           = G_OBJECT_CLASS (klass);
  class->dispose  = (GObjectFinalizeFunc) rubrica_app_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_app_finalize;
}


static void
rubrica_app_init(RubricaApp* self)
{
  RubricaDefaultCategories* cat = defcats;
  RubricaGroup* group;

  self->private = g_new(RubricaAppPrivate, 1);
    
  self->private->window     = NULL;

  self->private->active     = NULL;
  self->private->trash      = NULL;

  self->private->menu       = NULL;
  self->private->toolbar    = NULL;
  self->private->desk       = NULL;

  self->private->stack      = NULL;

  self->private->context_id = 0;
 

  /* add default groups 
   */
  self->private->set = rubrica_group_set_new();
  for(; cat->label; cat++) 
    {
      group = rubrica_group_new();
      
      g_object_set(group, "group-name", cat->label, "group-owner", "rubrica",
		   "group-pixmap", cat->pixmap, "enabled", FALSE, NULL);

      rubrica_group_set_add_group(self->private->set, group);
    }

  self->private->dispose_has_run = FALSE;
}



static void 
rubrica_app_dispose (RubricaApp* self)
{
  g_return_if_fail(IS_RUBRICA_APP(self));
  
  if (self->private->dispose_has_run)
    return;
  
  g_object_unref(self->private->window);
  g_free(self->private->menu);
  g_free(self->private->toolbar);
  g_free(self->private->desk);
    
  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_app_finalize (RubricaApp* self)
{
  g_return_if_fail(IS_RUBRICA_APP(self));
  
  g_free(self->private);
  self->private = NULL;
}


RubricaMenu*
rubrica_app_build_menu(GladeXML* gui, RubricaApp* app)
{
  RubricaMenu* menu = NULL;

  menu = g_malloc0(sizeof(RubricaMenu));
  if (!menu)
    g_error("\nCan't allocate memory");

  menu->menubar      = glade_xml_get_widget(gui, "menubar");
  menu->new          = glade_xml_get_widget(gui, "new");
  menu->open         = glade_xml_get_widget(gui, "open");
  menu->save         = glade_xml_get_widget(gui, "save");
  menu->saveas       = glade_xml_get_widget(gui, "saveas");
  menu->quit         = glade_xml_get_widget(gui, "quit");

  menu->cut          = glade_xml_get_widget(gui, "cut");
  menu->copy         = glade_xml_get_widget(gui, "copy");
  menu->paste        = glade_xml_get_widget(gui, "paste");
  menu->personal     = glade_xml_get_widget(gui, "personal");
  menu->company      = glade_xml_get_widget(gui, "company");
  menu->delete       = glade_xml_get_widget(gui, "delete");
  menu->modify       = glade_xml_get_widget(gui, "modify");
  menu->prefer       = glade_xml_get_widget(gui, "prefer");

  menu->about        = glade_xml_get_widget(gui, "about"); 

  /*     menu handler
  */ 
  g_signal_connect(G_OBJECT(menu->new), "activate",   
		   G_CALLBACK(on_new_cb), app);   
 
  g_signal_connect(G_OBJECT(menu->open), "activate",  
		   G_CALLBACK(on_open_cb), app);

  g_signal_connect(G_OBJECT(menu->save), "activate", 
		   G_CALLBACK(on_save_cb), app);

  g_signal_connect(G_OBJECT(menu->saveas), "activate", 
		   G_CALLBACK(on_saveas_cb), app);

  g_signal_connect(G_OBJECT(menu->cut), "activate", 
		   G_CALLBACK(on_cut_cb), NULL);

  g_signal_connect(G_OBJECT(menu->copy), "activate", 
		   G_CALLBACK(on_copy_cb), NULL);

  g_signal_connect(G_OBJECT(menu->paste), "activate", 
		   G_CALLBACK(on_paste_cb), NULL);

  g_signal_connect(G_OBJECT(menu->personal), "activate", 
		   G_CALLBACK(on_add_personal_cb), app);

  g_signal_connect(G_OBJECT(menu->company), "activate", 
		   G_CALLBACK(on_add_company_cb), app);

  g_signal_connect(G_OBJECT(menu->delete), "activate", 
		   G_CALLBACK(on_delete_card_cb), app);

  g_signal_connect(G_OBJECT(menu->modify), "activate", 
		   G_CALLBACK(on_modify_card_cb), app);

  g_signal_connect(G_OBJECT(menu->prefer), "activate", 
		   G_CALLBACK(on_prefenences_cb), app);

  g_signal_connect(G_OBJECT(menu->about), "activate", 
		   G_CALLBACK(on_info_cb), NULL);

  g_signal_connect(G_OBJECT(menu->quit), "activate", 
		   G_CALLBACK(on_quit_cb), app);

  return menu;
}


RubricaToolbar*
rubrica_app_build_toolbar(GladeXML* gui, RubricaApp* app)
{
  RubricaToolbar* toolbar = NULL;
  GtkMenu* menu = NULL;
  GtkWidget *personal;
  GtkWidget *company; 

  toolbar = g_malloc0(sizeof(RubricaToolbar));  
  if (!toolbar)
    g_error("\nCan't allocate memory");

  toolbar->handle  = glade_xml_get_widget(gui, "handle");
  toolbar->toolbar = glade_xml_get_widget(gui, "toolbar");
  toolbar->new     = glade_xml_get_widget(gui, "new_bt");
  toolbar->open    = glade_xml_get_widget(gui, "open_bt");
  toolbar->save    = glade_xml_get_widget(gui, "save_bt");
  toolbar->print   = glade_xml_get_widget(gui, "print_bt");
  toolbar->close   = glade_xml_get_widget(gui, "close_bt");

  toolbar->add     = glade_xml_get_widget(gui, "add_bt");
  toolbar->delete  = glade_xml_get_widget(gui, "delete_bt");
  toolbar->modify  = glade_xml_get_widget(gui, "modify_bt");

  toolbar->first   = glade_xml_get_widget(gui, "first_bt");
  toolbar->last    = glade_xml_get_widget(gui, "last_bt");
  toolbar->search  = glade_xml_get_widget(gui, "search_bt");
  toolbar->quit    = glade_xml_get_widget(gui, "quit_bt");  

  menu = (GtkMenu*) gtk_menu_new();
  
  personal = gtk_menu_item_new_with_mnemonic ("_Personal card");
  gtk_widget_show (personal);
  gtk_container_add (GTK_CONTAINER (menu), personal);
  
  company = gtk_menu_item_new_with_mnemonic ("_Company card");
  gtk_widget_show (company);
  gtk_container_add (GTK_CONTAINER (menu), company);
  
  gtk_widget_set_sensitive(company, FALSE);
  gtk_menu_tool_button_set_menu(GTK_MENU_TOOL_BUTTON(toolbar->add), 
				(GtkWidget*) menu);

  g_signal_connect (G_OBJECT(personal), "activate",
		    G_CALLBACK (on_add_personal_cb), app);
  
  g_signal_connect (G_OBJECT(company), "activate",
		    G_CALLBACK (on_add_company_cb), app);   
  
  g_signal_connect(G_OBJECT(toolbar->new), "clicked", 
		   G_CALLBACK(on_new_cb), app);

  g_signal_connect(G_OBJECT(toolbar->open), "clicked",
		   G_CALLBACK(on_open_cb), app);

  g_signal_connect(G_OBJECT(toolbar->save), "clicked",
		   G_CALLBACK(on_save_cb), app);
  
  g_signal_connect(G_OBJECT(toolbar->print), "clicked", 
		   G_CALLBACK(on_print_cb), app);

  g_signal_connect(G_OBJECT(toolbar->close), "clicked", 
		   G_CALLBACK(on_close_cb), app);

  g_signal_connect(G_OBJECT(toolbar->add), "clicked",
		   G_CALLBACK(on_add_personal_cb), app);
  
  g_signal_connect(G_OBJECT(toolbar->add), "show-menu",
		   G_CALLBACK(on_show_menu_cb), menu);
  
  g_signal_connect(G_OBJECT(toolbar->delete), "clicked", 
		   G_CALLBACK(on_delete_card_cb), app);

  g_signal_connect(G_OBJECT(toolbar->modify), "clicked", 
		   G_CALLBACK(on_modify_card_cb), app);

  g_signal_connect(G_OBJECT(toolbar->first), "clicked", 
		   G_CALLBACK(on_first_cb), app);

  g_signal_connect(G_OBJECT(toolbar->last), "clicked", 
		   G_CALLBACK(on_last_cb), app);

  g_signal_connect(G_OBJECT(toolbar->search), "clicked", 
		   G_CALLBACK(on_search_cb), app);

  g_signal_connect(G_OBJECT(toolbar->quit), "clicked", 
		   G_CALLBACK(on_quit_cb), app);

  return toolbar;
}



RubricaDesk*
rubrica_app_build_desk(GladeXML* gui, RubricaApp* app)
{
  RubricaDesk* desk = NULL;
  GtkTreeModel *model;
  GtkTreeSelection *selection;
  gint ctxid;

  desk = g_malloc0(sizeof(RubricaDesk));
  if (!desk)
    g_error("\nCan't allocate memory");
  
  desk->groupslist = glade_xml_get_widget(gui, "groupstree"); // lista gruppi
  desk->bookstree  = glade_xml_get_widget(gui, "bookstree");  // lista rubriche
  
  desk->notebook  = glade_xml_get_widget(gui, "cardsnotebook");
  
  desk->textview   = glade_xml_get_widget(gui, "textview");
  desk->previous   = glade_xml_get_widget(gui, "previous");
  desk->next       = glade_xml_get_widget(gui, "next");
  desk->statusbar  = glade_xml_get_widget(gui, "statusbar");

  ctxid = gtk_statusbar_get_context_id(GTK_STATUSBAR(desk->statusbar),
				       "RubricaContext");
  app->private->context_id = ctxid; 
  
  /*    init data view
   */
  rubrica_view_init_view (app, desk->textview);
  
  /*    Group's model
   */
  model = rubrica_model_new(GROUP_MODEL); 
  gtk_tree_view_set_model(GTK_TREE_VIEW(desk->groupslist), model);  
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(desk->groupslist));
  gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
  rubrica_model_add_columns (GTK_TREE_VIEW(desk->groupslist), 
			     APP_GROUP_COLUMNS);
  rubrica_model_populate_groups_view(GTK_TREE_VIEW(desk->groupslist), 
				     app->private->set);

  /*    Create addresses tree's branches and 
   */
  append_addresses_branches(app, GTK_TREE_VIEW(desk->bookstree));


  /*    Callbacks
  */
  g_signal_connect(G_OBJECT(desk->notebook), "switch-page",
		   G_CALLBACK(on_notebook_switch_page), app);  

  g_signal_connect(GTK_OBJECT (desk->groupslist), "button_press_event",
		   G_CALLBACK (on_rubrica_app_groups_event), app);
 
  g_signal_connect(G_OBJECT(desk->previous), "clicked", 
		   G_CALLBACK(on_prev_cb), app);

  g_signal_connect(G_OBJECT(desk->next), "clicked", 
		   G_CALLBACK(on_next_cb), app);
  
  return desk;
}



/*    Public
*/

/**
 * rubrica_app_new
 *
 * create a new #RubricaApp
 *
 * returns: a new allocated #RubricaApp
 */
RubricaApp* 
rubrica_app_new (void)
{
  RubricaApp* app;
  RubricaAdb* adb;
  GladeXML* gui;  

  app = g_object_new(rubrica_app_get_type(), NULL);

  gui = glade_xml_new (RUBRICA_GUI_DIR"/main/MainWindow.glade", NULL, NULL);
  if (!gui)
    g_error("Can't load gui");
  
  app->private->window  = glade_xml_get_widget(gui, "RubricaMainWindow");
  app->private->menu    = rubrica_app_build_menu(gui, app);
  app->private->toolbar = rubrica_app_build_toolbar(gui, app);
  app->private->desk    = rubrica_app_build_desk(gui, app);
  
  /*    Pixmaps in the statusbar
   */
  app->private->desk->smile = gtk_image_new();
  gtk_box_pack_start(GTK_BOX(app->private->desk->statusbar), 
		     app->private->desk->smile, FALSE, FALSE, 2);
  rubrica_app_enable_rank_pixmap(app, RUBRICA_RANK_NORMAL);
  gtk_widget_show(app->private->desk->smile);

  app->private->desk->lock = gtk_image_new();
  gtk_box_pack_start(GTK_BOX(app->private->desk->statusbar), 
		     app->private->desk->lock, FALSE, FALSE, 2);
  rubrica_app_enable_lock_pixmap(app, PIXMAP_LOCK);
  gtk_widget_show(app->private->desk->lock);
  
  gtk_widget_set_sensitive(app->private->desk->lock, FALSE);
  gtk_widget_set_sensitive(app->private->desk->smile, FALSE);
 
  /*    init Trash
   */  
  init_trash_branch(app);

  /*    Append an empty addressbook
  */
  adb = rubrica_adb_new();
  rubrica_app_append_addressbook(app, adb);
  
  gtk_tree_view_expand_all(GTK_TREE_VIEW(app->private->desk->bookstree));

  /*    Handle callbacks
  */
  g_signal_connect(G_OBJECT(app->private->set), "group_added", 
		   G_CALLBACK(on_rubrica_add_to_groupstree),
		   app->private->desk->groupslist);

  g_signal_connect(G_OBJECT(app->private->set), "group_removed", 
		   G_CALLBACK(on_rubrica_remove_from_groupstree),
		   app->private->desk->groupslist);

  g_signal_connect(G_OBJECT(app->private->window), "destroy", 
		   G_CALLBACK(on_quit_cb), app);
     
  g_object_unref(gui);
  
  return app;
}


/**
 * rubrica_app_free
 * @set: a #RubricaApp
 *
 * free memory owned by a #RubricaApp
 */
void
rubrica_app_free(RubricaApp* app)
{
  g_return_if_fail(IS_RUBRICA_APP(app));

  g_object_unref(app);   
}


void
rubrica_app_run(RubricaApp* app)
{
  g_return_if_fail(IS_RUBRICA_APP(app));
  
  gtk_widget_show_all(app->private->window);
}



/*   Get
*/
GtkWidget*  
rubrica_app_get_main_window (RubricaApp* app)
{
  g_return_val_if_fail(IS_RUBRICA_APP(app), NULL);
  
  return app->private->window;
}


RubricaAdb* 
rubrica_app_get_active_addressbook (RubricaApp* app)
{
  g_return_val_if_fail(IS_RUBRICA_APP(app), NULL);
  
  return app->private->active;
}


RubricaGroupSet* 
rubrica_app_get_active_user_groups (RubricaApp* app)
{
  g_return_val_if_fail(IS_RUBRICA_APP(app), NULL);

  return rubrica_adb_get_user_groups(app->private->active);
}


RubricaGroupSet* 
rubrica_app_get_groups (RubricaApp* app)
{
  g_return_val_if_fail(IS_RUBRICA_APP(app), NULL);
  
  return app->private->set;
}


GtkWidget*  
rubrica_app_get_textview (RubricaApp* app)
{
  g_return_val_if_fail(IS_RUBRICA_APP(app), NULL);

  return app->private->desk->textview;
}

GtkWidget* 
rubrica_app_get_groupstree(RubricaApp* app)
{
  g_return_val_if_fail(IS_RUBRICA_APP(app), NULL);

  return app->private->desk->groupslist;  
}


GtkWidget* 
rubrica_app_get_bookstree (RubricaApp* app)
{
  g_return_val_if_fail(IS_RUBRICA_APP(app), NULL);

  return app->private->desk->bookstree;  
}


gint             
rubrica_app_get_stack_len (RubricaApp* app)
{
  g_return_val_if_fail(IS_RUBRICA_APP(app), 0);
  
  return g_list_length(app->private->stack);
}


void 
rubrica_app_set_active_addressbook  (RubricaApp* app, RubricaAdb* adb)
{
  g_return_if_fail(IS_RUBRICA_APP(app));
  g_return_if_fail(IS_RUBRICA_ADB(adb));

  app->private->active = adb;
  
}



void 
rubrica_app_enable_lock_pixmap(RubricaApp* app, RubricaLockPixmap lock)
{
  RubricaDesk* desk;
  GdkPixbuf* pixbuf = NULL;  
  
  g_return_if_fail(IS_RUBRICA_APP(app));
 
  desk = app->private->desk;

  if (lock == PIXMAP_LOCK)
    pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR
					      "/main/lock.png",
					      20, 20, NULL); 
  else
    pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR
					      "/main/unlock.png",
					      20, 20, NULL);    
  if (pixbuf)
    {
      gtk_image_set_from_pixbuf (GTK_IMAGE (desk->lock), pixbuf);
      
      gdk_pixbuf_unref (pixbuf); 
    }  
}


void             
rubrica_app_enable_rank_pixmap (RubricaApp* app, RubricaRank rank)
{
  RubricaDesk* desk;
  GdkPixbuf* pixbuf = NULL;  

  g_return_if_fail(IS_RUBRICA_APP(app));
  
  desk = app->private->desk;
  
  switch(rank)
    {
    case RUBRICA_RANK_NONE:
      pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR
						"/main/rank1.png",
						20, 20, NULL);
      break;
      
    case RUBRICA_RANK_FEW:
      pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR
						"/main/rank2.png",
						20, 20, NULL);
      break;
      
    case RUBRICA_RANK_NORMAL:
      pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR
						"/main/rank3.png",
						20, 20, NULL);
      break;
      
    case RUBRICA_RANK_HIGHT:
      pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR
						"/main/rank4.png",
						20, 20, NULL);
      break;
      
    case RUBRICA_RANK_VERY_HIGHT:
      pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR
						"/main/rank5.png",
						20, 20, NULL);
      break;
      
    default:
      break;
    }
  
  if (pixbuf)
    {
      gtk_image_set_from_pixbuf (GTK_IMAGE (desk->smile), pixbuf);
      gdk_pixbuf_unref (pixbuf); 
    }  
}


void             
rubrica_app_show_page (RubricaApp* app, gint page)
{
  GtkWidget* notebook;

  g_return_if_fail(IS_RUBRICA_APP(app));

  notebook = app->private->desk->notebook;
  gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), page);   
}


void 
rubrica_app_move_card_to_trash (RubricaApp* app, RubricaCard* card)
{
  GtkTreeView* view;
  RubricaAbook* book;
  gchar* type;

  g_return_if_fail(IS_RUBRICA_APP(app));
  g_return_if_fail(IS_RUBRICA_CARD(card));

  book = rubrica_adb_get_addressbook(app->private->trash);
  rubrica_abook_add_card(book, card);

  view =  rubrica_adb_get_tree(app->private->trash);

  g_object_get(card, "card-type", &type, NULL);
  
  if (g_ascii_strcasecmp(type, "personal") == 0)
    append_personal_card(view, RUBRICA_PERSONAL_CARD(card));
  else
    append_company_card(view, RUBRICA_COMPANY_CARD(card));  
}



void             
rubrica_app_push (RubricaApp* app, gpointer data)
{
  g_return_if_fail(IS_RUBRICA_APP(app));
  
  app->private->stack = g_list_append(app->private->stack, data);
}


gpointer         
rubrica_app_pop (RubricaApp* app)
{
  GList* link = NULL;
  gpointer data = NULL;

  g_return_val_if_fail(IS_RUBRICA_APP(app), NULL);
  
  link = g_list_last(app->private->stack);
  
  if (link)
    {
      data = link->data;
      app->private->stack = g_list_remove_link(app->private->stack, link);
      
      g_list_free_1(link);
    }
  
  return data;
}




void             
rubrica_app_notify (RubricaApp* app, gchar* msg)
{
  g_return_if_fail(IS_RUBRICA_APP(app));

  gtk_statusbar_push(GTK_STATUSBAR(app->private->desk->statusbar), 
		     app->private->context_id, msg);
}





void 
rubrica_app_append_addressbook (RubricaApp* app, RubricaAdb* adb)
{
  RubricaAbook    *book;
  RubricaGroupSet *usergroups = NULL;
  RubricaGroupSet *set = NULL;
  GtkWidget *child;
  GtkWidget *label;
  gchar *name = NULL;
  gint page, items = 0;

  g_return_if_fail(IS_RUBRICA_APP(app));
  g_return_if_fail(IS_RUBRICA_ADB(adb));

  book       = rubrica_adb_get_addressbook(adb);
  usergroups = rubrica_adb_get_user_groups(adb);
  set        = rubrica_app_get_groups(app);

  /* merge default group and user group, then update groups view 
   */
  if (usergroups)
    {
      GtkTreeView* view = GTK_TREE_VIEW(app->private->desk->groupslist);

      rubrica_model_update_groups_view(view, usergroups);
      rubrica_group_set_merge_sets(set, usergroups);
    }  

  /* if addressbook isn't empty */
  if ((name = g_strdup(g_basename(rubrica_adb_get_name(adb)))))
    label = gtk_label_new(name);
  else
    label = gtk_label_new("");
  items = rubrica_abook_get_items(book);

  child = cards_list_new(app, adb);
  g_object_set_data(G_OBJECT(child), "Addressbook", adb);  

  /*    append new card's list to notebook
  */
  page = gtk_notebook_append_page(GTK_NOTEBOOK(app->private->desk->notebook), 
				  child, label);  
  gtk_notebook_set_current_page(GTK_NOTEBOOK(app->private->desk->notebook),
				page);

  update_addresses_tree(app, name, page);
  
  if (items)
    {
      gpointer card;
      
      rubrica_abook_reset_book(book);
      card = rubrica_abook_get_card(book);
      for (; card; card = rubrica_abook_get_next_card(book))
	rubrica_app_add_card_to_view(app, card);  

      gtk_widget_set_sensitive(app->private->desk->lock, TRUE);
      gtk_widget_set_sensitive(app->private->desk->smile, TRUE);      
    }
  else
    {
      gtk_widget_set_sensitive(app->private->desk->lock, FALSE);
      gtk_widget_set_sensitive(app->private->desk->smile, FALSE);      
    }
  

  g_signal_connect(G_OBJECT(book), "card_added",
		   G_CALLBACK(on_card_added), app);

  g_signal_connect(G_OBJECT(book), "card_removed",
		   G_CALLBACK(on_card_removed), app);

  g_signal_connect(G_OBJECT(book), "card_replaced",
		   G_CALLBACK(on_card_modified), app);
}




void 
rubrica_app_add_card_to_view(RubricaApp* app, RubricaCard* card)
{
  RubricaAdb  *adb;
  GtkTreeView *view =NULL;
  gchar* type;
  
  g_return_if_fail(IS_RUBRICA_APP(app));
  g_return_if_fail(IS_RUBRICA_CARD(card));

  adb   = rubrica_app_get_active_addressbook (app);
  view  = rubrica_adb_get_tree(adb);

  gtk_widget_set_sensitive(app->private->desk->lock, TRUE);
  gtk_widget_set_sensitive(app->private->desk->smile, TRUE);

  g_object_get(card, "card-type", &type, NULL);
  
  if (g_ascii_strcasecmp(type, "personal") == 0)
    append_personal_card(view, RUBRICA_PERSONAL_CARD(card));
  else
    append_company_card(view, RUBRICA_COMPANY_CARD(card));
}
 

void 
rubrica_app_update_cards_view_on_delete (RubricaApp* app)
{
  RubricaAdb *adb;
  GtkTreeView  *tree;
  GtkTreeModel *model;
  GtkTreeIter   iter;  
  GtkTreeSelection *selection;

  adb       = rubrica_app_get_active_addressbook(app);
  tree      = rubrica_adb_get_tree(adb);
  model     = gtk_tree_view_get_model(GTK_TREE_VIEW(tree)); 
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));

  if (gtk_tree_selection_get_selected(selection, NULL, &iter))
    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
}


void 
rubrica_app_update_cards_view_on_cut    (RubricaApp* app)
{
}


void 
rubrica_app_update_cards_view_on_paste  (RubricaApp* app)
{

}


void
rubrica_app_update_bookstree_view  (RubricaApp* app)
{
  RubricaAdb *adb;
  RubricaAbook *book;
  GtkTreeView  *tree;
  GtkTreeModel *model;
  GtkTreeIter   iter, parent;  
  GtkTreeSelection *selection;
  gint items;
  
  adb   = rubrica_app_get_active_addressbook(app);
  book  = rubrica_adb_get_addressbook(adb);
  items = rubrica_abook_get_items(book);

  tree      = (GtkTreeView*) app->private->desk->bookstree;
  model     = gtk_tree_view_get_model(GTK_TREE_VIEW(tree)); 
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
  
  if (gtk_tree_selection_get_selected(selection, NULL, &iter))
  {
    GtkTreePath *path;
    GdkPixbuf* pixbuf = NULL;
    gchar* name = NULL;
    gchar *str;
    gint page, index;
    
    gtk_tree_model_get(model, &iter, 
		       ADDRESSBOOK_FILENAME_COLUMN, &name,
		       ADDRESSBOOK_PAGE_COLUMN,     &page,
		       -1);
    
    if (page < 0) 
      return;
    
    str = g_strdup_printf("%d", items);
    pixbuf = rubrica_utils_pixbuf_new ("ledred.png"); 
    
    path  = gtk_tree_model_get_path(model, &iter);       
    index = gtk_tree_path_get_indices(path)[0];

    gtk_tree_model_get_iter(model, &iter, path);
    if (gtk_tree_path_up(path))
      gtk_tree_model_get_iter(model, &parent, path);
    
    gtk_tree_store_remove(GTK_TREE_STORE(model), &iter);
    gtk_tree_path_free(path);
 
    gtk_tree_store_insert(GTK_TREE_STORE(model), &iter, &parent, index);
    gtk_tree_store_set(GTK_TREE_STORE(model), &iter,   
		       ADDRESSBOOK_PIXMAP_COLUMN,   pixbuf,
		       ADDRESSBOOK_FILENAME_COLUMN, name,
		       ADDRESSBOOK_PAGE_COLUMN,     page,
		       -1);		       
  }
}
