/*
 *   file: form_contact.c
 *  
 *  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <libral.h>

#include "app.h"
#include "form.h"
#include "form_contact.h"
#include "app.h"
#include "groups_cb.h"
#include "models.h"
#include "types.h"
#include "utils.h"


struct _RubricaFormContactPrivate {
  RubricaGroupSet* set;

  GtkWidget* fn;
  GtkWidget* sn;
  GtkWidget* ln;
  GtkWidget* nick;
  GtkWidget* prof;
  GtkWidget* title;
  GtkWidget* prefix;
  GtkWidget* birth;
  GtkWidget* birth_bt;
  GtkWidget* photo_entry;
  GtkWidget* photo_bt; 
  GtkWidget* photo;
  
  gboolean dispose_has_run;
};


static void rubrica_form_contact_class_init (RubricaFormContactClass* klass);
static void rubrica_form_contact_init       (RubricaFormContact* obj);

static void rubrica_form_contact_dispose    (RubricaFormContact* obj);
static void rubrica_form_contact_finalize   (RubricaFormContact* obj);



static void form_contact_extract_data (RubricaFormContact* form,
				       RubricaCard *card);
static void form_contact_display_data (RubricaFormContact* form, 
				       RubricaCard *card);
static void form_contact_clean_form   (RubricaFormContact* form, 
				       RubricaCard *card);


static RubricaFormClass* parent_class;


GType
rubrica_form_contact_get_type()
{
  static GType form_contact_type = 0;
  
  if (!form_contact_type)
    {
      static const GTypeInfo form_contact_info =
	{
	  sizeof(RubricaFormContactClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_form_contact_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaFormContact),
	  0,
	  (GInstanceInitFunc) rubrica_form_contact_init
	};

      form_contact_type = g_type_register_static (RUBRICA_FORM_TYPE, 
						  "RubricaFormContact",
						  &form_contact_info, 0);
    }
  
  return form_contact_type;
}


static void
rubrica_form_contact_class_init(RubricaFormContactClass* klass)
{
  GObjectClass *class;
 
  class  = G_OBJECT_CLASS (klass);
  class->dispose  = (GObjectFinalizeFunc) rubrica_form_contact_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_form_contact_finalize;

  parent_class = RUBRICA_FORM_CLASS(klass);
  parent_class->extract_data = (gpointer) form_contact_extract_data;
  parent_class->display_data = (gpointer) form_contact_display_data;
  parent_class->clean_form   = (gpointer) form_contact_clean_form;
}


static void
rubrica_form_contact_init(RubricaFormContact* self)
{
  self->private = g_new(RubricaFormContactPrivate, 1);

  self->private->fn          = NULL;
  self->private->sn          = NULL;
  self->private->ln          = NULL;
  self->private->nick        = NULL;
  self->private->prof        = NULL;
  self->private->title       = NULL;
  self->private->prefix      = NULL;
  self->private->birth       = NULL;
  self->private->birth_bt    = NULL;
  self->private->photo_entry = NULL;
  self->private->photo_bt    = NULL;
  self->private->photo       = NULL;

  self->private->dispose_has_run = FALSE;
}

 

static void 
rubrica_form_contact_dispose (RubricaFormContact* self)
{
  g_return_if_fail(IS_RUBRICA_FORM(self));
  
  if (self->private->dispose_has_run)
    return;
  
  g_object_unref(self->private->fn);
  g_object_unref(self->private->sn);
  g_object_unref(self->private->ln);
  g_object_unref(self->private->nick);
  g_object_unref(self->private->prof);
  g_object_unref(self->private->prefix);
  g_object_unref(self->private->title);
  g_object_unref(self->private->birth);
  g_object_unref(self->private->photo);
  g_object_unref(self->private->birth_bt);
  g_object_unref(self->private->photo_entry);
  g_object_unref(self->private->photo_bt);
  
  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_form_contact_finalize (RubricaFormContact* self)
{
  g_return_if_fail(IS_RUBRICA_FORM(self));
  
  g_free(self->private);
  self->private = NULL;
}



static void 
form_contact_extract_data (RubricaFormContact* form, RubricaCard *card)
{
  RubricaContact* contact;
  gchar *date = NULL;
  gint day = -1, month = -1, year = -1;

  g_return_if_fail(IS_RUBRICA_CARD(card));

  contact = rubrica_contact_new();

  date = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->private->birth)));
  if (g_ascii_strcasecmp(date, "unknown") != 0)
    {      
      day   = (gint) g_object_get_data(G_OBJECT(form->private->birth), "day");
      month = (gint) g_object_get_data(G_OBJECT(form->private->birth), "month") + 1;
      year  = (gint) g_object_get_data(G_OBJECT(form->private->birth), "year");
    }
  g_free(date);

  g_object_set(contact,
	       "first-name", gtk_entry_get_text(GTK_ENTRY(form->private->fn)),
	       "middle-name",gtk_entry_get_text(GTK_ENTRY(form->private->sn)),
	       "last-name",  gtk_entry_get_text(GTK_ENTRY(form->private->ln)),
	       "nick-name",  gtk_entry_get_text(GTK_ENTRY(form->private->nick)),
	       "profession", gtk_entry_get_text(GTK_ENTRY(form->private->prof)),
	       "title",      gtk_entry_get_text(GTK_ENTRY(form->private->title)),
	       "prefix",     gtk_entry_get_text(GTK_ENTRY(form->private->prefix)),
	       "photo",      gtk_entry_get_text(GTK_ENTRY(form->private->photo_entry)),
	       NULL);

  rubrica_contact_set_birthday(contact, day, month, year);
  
  rubrica_personal_card_set_contact(RUBRICA_PERSONAL_CARD(card), contact);
}


static void 
form_contact_display_data (RubricaFormContact* form, RubricaCard *card)
{
  RubricaContact* contact;

  GtkImage  *image;
  GdkPixbuf *pixbuf;
 
  gchar *first, *middle, *last, *nick, *prof, *pref;
  gchar *title, *photo, *day, *month, *year;

  contact = rubrica_personal_card_get_contact (RUBRICA_PERSONAL_CARD(card));

  g_object_get(contact, "first-name", &first, "middle-name", &middle, 
	       "last-name", &last, "nick-name", &nick, 
	       "profession", &prof, "prefix", &pref,
	       "title", &title, "photo", &photo, NULL);
  day   = rubrica_contact_get_birth_day(contact);
  month = rubrica_contact_get_birth_month(contact);
  year  = rubrica_contact_get_birth_year(contact);  

  if (first)  gtk_entry_set_text(GTK_ENTRY(form->private->fn), first);
  if (middle) gtk_entry_set_text(GTK_ENTRY(form->private->sn), middle);
  if (last)   gtk_entry_set_text(GTK_ENTRY(form->private->ln), last);
  if (nick)   gtk_entry_set_text(GTK_ENTRY(form->private->nick), nick);
  if (prof)   gtk_entry_set_text(GTK_ENTRY(form->private->prof), prof);
  if (title)  gtk_entry_set_text(GTK_ENTRY(form->private->title), title);
  if (pref)   gtk_entry_set_text(GTK_ENTRY(form->private->prefix), pref);
  if (photo)  gtk_entry_set_text(GTK_ENTRY(form->private->photo_entry), photo);
  /* compleanno */  

  image  = (GtkImage*) form->private->photo;
  pixbuf = gdk_pixbuf_new_from_file_at_size (photo, 164, 164, NULL);
  gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
}


static void 
form_contact_clean_form   (RubricaFormContact* form, RubricaCard *card)
{

}



/*    Callbacks
*/
void
form_add_photo(GtkWidget* button, gpointer data)
{
  RubricaFormContact* form = (RubricaFormContact*) data;
  GtkImage* image;
  GdkPixbuf *pixbuf;
  gchar* uri;

  image = (GtkImage*) form->private->photo;

  rubrica_utils_load_pixmap(button, image);
  uri = g_object_get_data(G_OBJECT(button), "uri");

  pixbuf = gdk_pixbuf_new_from_file_at_size (uri, 164, 164, NULL);
  gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
  gtk_entry_set_text(GTK_ENTRY(form->private->photo_entry), uri);
}

/*   ***************************** Public *****************************
*/


/**
 * rubrica_form_contact_new
 *
 * create a #RubricaFormContact
 *
 * Returns: a new #RubricaFormContact
 */
RubricaFormContact*
rubrica_form_contact_new(RubricaApp* app, GladeXML *gui)
{
  RubricaFormContact* form;

  form = g_object_new(rubrica_form_contact_get_type(), NULL);
  
  form->private->fn          = glade_xml_get_widget(gui, "fn");
  form->private->nick        = glade_xml_get_widget(gui, "nick");
  form->private->sn          = glade_xml_get_widget(gui, "sn");
  form->private->ln          = glade_xml_get_widget(gui, "ln");
  form->private->prof        = glade_xml_get_widget(gui, "prof");
  form->private->title       = glade_xml_get_widget(gui, "title");
  form->private->prefix      = glade_xml_get_widget(gui, "prefix");
  form->private->birth       = glade_xml_get_widget(gui, "birth");
  form->private->birth_bt    = glade_xml_get_widget(gui, "birth_bt");
  form->private->photo_entry = glade_xml_get_widget(gui, "photo_entry");
  form->private->photo_bt    = glade_xml_get_widget(gui, "photo_bt");
  form->private->photo       = glade_xml_get_widget(gui, "photo");
  
  /*     calendar buttons 
   */
  g_signal_connect (G_OBJECT (form->private->birth_bt), "clicked", 
		    G_CALLBACK (rubrica_utils_calendar_ui), 
		    form->private->birth); 

  g_signal_connect (G_OBJECT (form->private->photo_bt), "clicked",
		    G_CALLBACK(form_add_photo), form);
  
  
  return form;
}


/**
 * rubrica_form_contact_free
 * @form: a #RubricaFormContact
 * 
 * free the #RubricaFormContact
 */
void
rubrica_form_contact_free(RubricaFormContact* form)
{
  g_return_if_fail(IS_RUBRICA_FORM(form));

  g_object_unref(form);   
}
